@@ -1,5 +1,5 @@
-use self::super::{AluOperationShiftOrRotateDirection, AluOperationShiftOrRotateType, InstructionJumpCondition, InstructionPortDirection,
-                  InstructionMadrDirection, InstructionStckDirection, InstructionRegisterPair, AluOperation, Instruction};
+use self::super::{InstructionLoadImmediateWideRegisterPair, AluOperationShiftOrRotateDirection, AluOperationShiftOrRotateType, InstructionJumpCondition,
+                  InstructionPortDirection, InstructionMadrDirection, InstructionStckDirection, InstructionRegisterPair, AluOperation, Instruction};
 use self::super::super::GeneralPurposeRegisterBank;
 use std::fmt;
 
@@ -19,15 +19,16 @@ impl<'a> fmt::Display for DisplayInstruction<'a> {
     fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
         match self.instr {
             Instruction::Reserved(raw) => write!(f, "{:#06b}_{:04b}", (raw & 0b1111_0000) >> 4, raw & 0b0000_1111),
-            Instruction::Madr { d, r } => write!(f, "MADR {} {}", d, r),
+            Instruction::LoadImmediateByte { rrr } => write!(f, "LOAD IMM BYTE {}", self.registers[*rrr as usize].letter()),
+            Instruction::LoadIndirect { rrr } => write!(f, "LOAD IND {}", self.registers[*rrr as usize].letter()),
+            Instruction::LoadImmediateWide { rr } => write!(f, "LOAD IMM WIDE {}", rr),
             Instruction::Jump(cond) => cond.fmt(f),
-            Instruction::LoadImmediate { aaa } => write!(f, "LOAD IMM {}", self.registers[*aaa as usize].letter()),
-            Instruction::LoadIndirect { aaa } => write!(f, "LOAD IND {}", self.registers[*aaa as usize].letter()),
-            Instruction::Save { aaa } => write!(f, "SAVE {}", self.registers[*aaa as usize].letter()),
+            Instruction::Save { rrr } => write!(f, "SAVE {}", self.registers[*rrr as usize].letter()),
             Instruction::Alu(op) => write!(f, "ALU {}", op),
-            Instruction::Move { aaa, bbb } => write!(f, "MOVE {} {}", self.registers[*aaa as usize].letter(), self.registers[*bbb as usize].letter()),
-            Instruction::Port { d, aaa } => write!(f, "PORT {} {}", d, self.registers[*aaa as usize].letter()),
-            Instruction::Comp { aaa } => write!(f, "COMP {}", self.registers[*aaa as usize].letter()),
+            Instruction::Move { qqq, rrr } => write!(f, "MOVE {} {}", self.registers[*qqq as usize].letter(), self.registers[*rrr as usize].letter()),
+            Instruction::Madr { d, r } => write!(f, "MADR {} {}", d, r),
+            Instruction::Port { d, rrr } => write!(f, "PORT {} {}", d, self.registers[*rrr as usize].letter()),
+            Instruction::Comp { rrr } => write!(f, "COMP {}", self.registers[*rrr as usize].letter()),
             Instruction::Stck { d, r } => write!(f, "STCK {} {}", d, r),
             Instruction::Clrf => f.write_str("CLRF"),
             Instruction::Halt => f.write_str("HALT"),
@@ -82,6 +83,18 @@ impl fmt::Display for InstructionStckDirection {
 }
 
 
+impl fmt::Display for InstructionLoadImmediateWideRegisterPair {
+    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
+        match self {
+            InstructionLoadImmediateWideRegisterPair::Ab => f.write_str("A&B"),
+            InstructionLoadImmediateWideRegisterPair::Cd => f.write_str("C&D"),
+            InstructionLoadImmediateWideRegisterPair::Xy => f.write_str("X&Y"),
+            InstructionLoadImmediateWideRegisterPair::Adr => f.write_str("ADR"),
+        }
+    }
+}
+
+
 impl fmt::Display for InstructionRegisterPair {
     fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
         match self {
