@@ -1,14 +1,24 @@
-use pir_8_emu::isa::instruction::{AluOperationShiftOrRotateDirection, AluOperationShiftOrRotateType, InstructionJumpCondition, InstructionPortDirection,
-                                  InstructionMadrDirection, InstructionStckDirection, InstructionRegisterPair, AluOperation, Instruction};
+use pir_8_emu::isa::instruction::{InstructionLoadImmediateWideRegisterPair, AluOperationShiftOrRotateDirection, AluOperationShiftOrRotateType,
+                                  InstructionJumpCondition, InstructionPortDirection, InstructionMadrDirection, InstructionStckDirection,
+                                  InstructionRegisterPair, AluOperation, Instruction};
 
 
 #[test]
-fn madr() {
-    for &d in &[InstructionMadrDirection::Write, InstructionMadrDirection::Read] {
-        for &r in &[InstructionRegisterPair::Ab, InstructionRegisterPair::Cd] {
-            assert!(Instruction::Madr { d: d, r: r }.is_valid());
-        }
-    }
+fn load_immediate_byte() {
+    single_register(|r| Instruction::LoadImmediateByte { rrr: r });
+}
+
+#[test]
+fn load_indirect() {
+    single_register(|r| Instruction::LoadIndirect { rrr: r });
+}
+
+#[test]
+fn load_immediate_wide() {
+    assert!(Instruction::LoadImmediateWide { rr: InstructionLoadImmediateWideRegisterPair::Ab }.is_valid());
+    assert!(Instruction::LoadImmediateWide { rr: InstructionLoadImmediateWideRegisterPair::Cd }.is_valid());
+    assert!(Instruction::LoadImmediateWide { rr: InstructionLoadImmediateWideRegisterPair::Xy }.is_valid());
+    assert!(Instruction::LoadImmediateWide { rr: InstructionLoadImmediateWideRegisterPair::Adr }.is_valid());
 }
 
 #[test]
@@ -25,19 +35,9 @@ fn jump() {
     }
 }
 
-#[test]
-fn load_immediate() {
-    single_register(|r| Instruction::LoadImmediate { aaa: r });
-}
-
-#[test]
-fn load_indirect() {
-    single_register(|r| Instruction::LoadIndirect { aaa: r });
-}
-
 #[test]
 fn save() {
-    single_register(|r| Instruction::Save { aaa: r });
+    single_register(|r| Instruction::Save { rrr: r });
 }
 
 #[test]
@@ -66,36 +66,45 @@ fn alu_shift_or_rotate() {
 
 #[test]
 fn move_() {
-    for aaa in 0..=0b111 {
-        for bbb in 0..=0b111 {
+    for qqq in 0..=0b111 {
+        for rrr in 0..=0b111 {
             assert!(Instruction::Move {
-                    aaa: aaa,
-                    bbb: bbb,
+                    qqq: qqq,
+                    rrr: rrr,
                 }
                 .is_valid());
         }
     }
 }
 
+#[test]
+fn madr() {
+    for &d in &[InstructionMadrDirection::Write, InstructionMadrDirection::Read] {
+        for &r in &[InstructionRegisterPair::Ab, InstructionRegisterPair::Cd] {
+            assert!(Instruction::Madr { d: d, r: r }.is_valid());
+        }
+    }
+}
+
 #[test]
 fn port() {
     single_register(|r| {
         Instruction::Port {
             d: InstructionPortDirection::In,
-            aaa: r,
+            rrr: r,
         }
     });
     single_register(|r| {
         Instruction::Port {
             d: InstructionPortDirection::Out,
-            aaa: r,
+            rrr: r,
         }
     });
 }
 
 #[test]
 fn comp() {
-    single_register(|r| Instruction::Comp { aaa: r });
+    single_register(|r| Instruction::Comp { rrr: r });
 }
 
 #[test]
